﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static CharaEdit40.Define;

namespace CharaEdit40
{
    // -------------------------------------------------------------
    // 定数
    // 
    public class Define
    {
        public const int CHIP_SIZE = 8;                 // 1文字の縦横ドット数
        public const int TEXT_WIDTH = 40;               // 1行の文字数
        public const int TEXT_HEIGHT = 25;              // マップ最大行数
        public const int ELEMENT_WIDTH = 0;             // ステージデータの幅の要素番号
        public const int ELEMENT_HEIGHT = 1;            // ステージデータの高さの要素番号
        public const int SELICON_WIDTH = 20;            // 選択アイコンの幅
        public const int SELICON_HEIGHT = 20;           // 選択アイコンの高さ
        public const int THUMBNAIL_WIDTH = 80;          // サムネイル画像横サイズ
        public const int ATB_BLACK = 0x00;              // アトリビュート：黒色 0
        public const int ATB_BLUE = 0x20;               // アトリビュート：青色 1
        public const int ATB_RED = 0x40;                // アトリビュート：赤色 2
        public const int ATB_MAGENTA = 0x60;            // アトリビュート：紫色 3
        public const int ATB_GREEN = 0x80;              // アトリビュート：緑色 4
        public const int ATB_CYAN = 0xA0;               // アトリビュート：水色 5
        public const int ATB_YELLOW = 0xC0;             // アトリビュート：黄色 6
        public const int ATB_WHITE = 0xE0;              // アトリビュート：白色 7
        public const int ATB_TRANSPARENT = 0x10;        // アトリビュート：指定なし
        public const int ATB_NORMAL = 0x00;             // 拡張アトリビュート：なし
        public const int ATB_OVERLINE = 0x01;           // 拡張アトリビュート：上線
        public const int ATB_UNDERLINE = 0x02;          // 拡張アトリビュート：下線
        public const int ATB_REVERSE = 0x04;            // 拡張アトリビュート：反転
        public enum Cmd                                 // 現在選択されているコマンド
        {
            Pen,            // ペン：待機中
            PenOn,          // ペン：キャラ文字を連続配置中
            Select,         // 選択：待機中
            SelectOn,       // 選択：選択枠変更中
            SelectMove,     // 選択：選択枠移動中
            Move,           // 移動：待機中
            MoveOn,         // 移動：位置変更中
            Text,           // テキスト：入力中
            DlgPattern,     // パターンダイアログ表示
        }
        public enum ExColor { Black, Blue, Red, Magenta, Green, Cyan, Yellow, White, Transparent = -1 };
    }

    // -------------------------------------------------------------
    // キャラクタ文字
    //
    [Serializable()]
    public struct Chara
    {
        public int code;
        public ExColor color;
        public int atrb;
        public Chara(int cd = 0, ExColor clr = ExColor.Transparent, int exa = ATB_NORMAL)
        {
            code = cd;
            color = clr;
            atrb = exa;
        }
        public Chara(Chara src) : this(src.code, src.color, src.atrb) { }
    }

    // -------------------------------------------------------------
    // レイヤーテーブル
    // 
    [Serializable()]
    public class Layer
    {
        public bool isShow = true;          // 見えるか
        public string label = string.Empty; // ラベル名
        public Chara[,] chrTable = null;    // キャラクタ文字配置テーブル
        public Image image = null;          // 表示イメージ
        public Image thumbnail = null;      // サムネイルイメージ
        public Layer() { }
        public Layer(Size sz) : this(sz.Width, sz.Height) { }
        public Layer(int w, int h)
        {
            chrTable = new Chara[w, h];
            Size sz = MainForm.form.GetThumbSize();
            thumbnail = new Bitmap(sz.Width, sz.Height);
            image = new Bitmap(w * CHIP_SIZE, h * CHIP_SIZE);
            Chara nullCode = new Chara(0, ExColor.White);
            for (int y = 0; y < h; ++y)
            {
                for (int x = 0; x < w; ++x)
                {
                    chrTable[x, y] = nullCode;
                }
            }
        }
        public Layer(Layer src) : this(src.chrTable.GetLength(ELEMENT_WIDTH), src.chrTable.GetLength(ELEMENT_HEIGHT))
        {
            isShow = src.isShow;
            label = src.label;
            for (int y = 0; y < src.chrTable.GetLength(ELEMENT_HEIGHT); ++y)
            {
                for (int x = 0; x < src.chrTable.GetLength(ELEMENT_WIDTH); ++x)
                {
                    chrTable[x, y] = src.chrTable[x, y];
                }
            }
            if (image != null) image.Dispose();
            image = new Bitmap(src.image);
            if (thumbnail != null) thumbnail.Dispose();
            thumbnail = new Bitmap(src.thumbnail);
        }
    }

    // -------------------------------------------------------------
    // ブロックデータ
    //
    public class Block : Layer
    {
        public Rectangle range;             // 表示範囲
        public Point offsetPos;             // オフセット座標
        public int layer = -1;              // 対象レイヤー番号
        public Block() { }
        public Block(int n, Rectangle rc) : this(n, rc.Location, rc.Size) { }
        public Block(int n, int x, int y, int w, int h) : this(n, new Point(x, y), new Size(w, h)) { }
        public Block(int n, Point pos, Size sz) : base(sz)
        {
            layer = n;
            offsetPos = pos;
            range = new Rectangle(pos, sz);
        }
        public Block(Block src) : base(src)
        {
            layer = src.layer;
            offsetPos = src.offsetPos;
            range = src.range;
        }
    }

    // -------------------------------------------------------------
    // データ保存クラス
    //
    [Serializable()]
    public class SaveData
    {
        public List<Layer> lsCharas { get; set; }
        public int editLayer { get; set; }
    }

    // -------------------------------------------------------------
    // アンドゥ保存クラス
    //
    public class UndoData : SaveData
    {
        public string filename { get; set; }
        public Point posCursor { get; set; }
    }

    // -------------------------------------------------------------
    // ダブルバッファでちらつきをおさえた ListView継承クラス
    // 
    class BufferedListView : ListView
    {
        public BufferedListView() { DoubleBuffered = true; }
        protected override bool DoubleBuffered { get { return true; } set { } }
    }

}
